package com.qr.flutter_toolkit;

import android.app.Activity;
import android.app.Application;
import android.content.Intent;
import android.os.Bundle;

import com.qr.flutter_toolkit.camera.CameraHandler;
import com.qr.flutter_toolkit.gps_location.GPSLocationHandler;
import com.qr.flutter_toolkit.package_info.PackageInfoHandler;
import com.qr.flutter_toolkit.permission.PermissionHandler;
import com.qr.flutter_toolkit.path_provider.PathProviderHandler;
import com.qr.flutter_toolkit.video_thumbnail.VideoThumbnailHandler;

import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.PluginRegistry;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/**
 * FlutterToolkitPlugin
 */
public class FlutterToolkitPlugin {
    private final static String TOOLKIT_PATH_PROVIDER_METHODS = "com.qr.flutter_toolkit/path_provider/methods";
    private final static String TOOLKIT_PACKAGE_INFO_METHODS = "com.qr.flutter_toolkit/package_info/methods";
    private final static String TOOLKIT_GEOLOCATION_METHODS = "com.qr.flutter_toolkit/gps_location/methods";
    private final static String TOOLKIT_GEOLOCATION_EVENTS = "com.qr.flutter_toolkit/gps_location/events";
    private final static String TOOLKIT_PERMISSION_METHODS = "com.qr.flutter_toolkit/permission/methods";
    private final static String TOOLKIT_CAMERA_METHODS = "com.qr.flutter_toolkit/camera/methods";
    private final static String TOOLKIT_CAMERA_EVENTS = "com.qr.flutter_toolkit/camera/events";
    private final static String TOOLKIT_VIDEO_THUMBNAIL_METHODS = "com.qr.flutter_toolkit/camera/methods";

    /**
     * Plugin registration.
     */
    public static void registerWith(Registrar registrar) {
        MethodChannel pathProviderMethodChannel = new MethodChannel(registrar.messenger(), TOOLKIT_PATH_PROVIDER_METHODS);
        pathProviderMethodChannel.setMethodCallHandler(new PathProviderHandler(registrar));

        MethodChannel packageInfoMethodChannel = new MethodChannel(registrar.messenger(), TOOLKIT_PACKAGE_INFO_METHODS);
        packageInfoMethodChannel.setMethodCallHandler(new PackageInfoHandler(registrar));

        MethodChannel geolocationMethodChannel = new MethodChannel(registrar.messenger(), TOOLKIT_GEOLOCATION_METHODS);
        EventChannel geolocationEventChannel = new EventChannel(registrar.messenger(), TOOLKIT_GEOLOCATION_EVENTS);
        GPSLocationHandler GPSLocationHandler = new GPSLocationHandler(registrar);
        geolocationMethodChannel.setMethodCallHandler(GPSLocationHandler);
        geolocationEventChannel.setStreamHandler(GPSLocationHandler);

        MethodChannel permissionMethodChannel = new MethodChannel(registrar.messenger(), TOOLKIT_PERMISSION_METHODS);
        PermissionHandler permissionHandler = new PermissionHandler(registrar);
        permissionMethodChannel.setMethodCallHandler(permissionHandler);

        MethodChannel cameraMethodChannel = new MethodChannel(registrar.messenger(), TOOLKIT_CAMERA_METHODS);
        CameraHandler cameraHandler = new CameraHandler(registrar);
        cameraMethodChannel.setMethodCallHandler(cameraHandler);

        MethodChannel videoThumbnailMethodChannel = new MethodChannel(registrar.messenger(), TOOLKIT_VIDEO_THUMBNAIL_METHODS);
        VideoThumbnailHandler videoThumbnailHandler = new VideoThumbnailHandler(registrar);
        videoThumbnailMethodChannel.setMethodCallHandler(videoThumbnailHandler);

        int hashCode = registrar.activity().hashCode();
        registrar.activity().getApplication().registerActivityLifecycleCallbacks(new Application.ActivityLifecycleCallbacks() {
            @Override
            public void onActivityCreated(Activity activity, Bundle savedInstanceState) {
                if (activity.hashCode() == hashCode) {

                }
            }

            @Override
            public void onActivityStarted(Activity activity) {
                if (activity.hashCode() == hashCode) {

                }
            }

            @Override
            public void onActivityResumed(Activity activity) {
                if (activity.hashCode() == hashCode) {
                    cameraHandler.onResumed();
                }
            }

            @Override
            public void onActivityPaused(Activity activity) {
                if (activity.hashCode() == hashCode) {
                    cameraHandler.onPaused();
                }
            }

            @Override
            public void onActivityStopped(Activity activity) {
                if (activity.hashCode() == hashCode) {

                }
            }

            @Override
            public void onActivitySaveInstanceState(Activity activity, Bundle outState) {
                if (activity.hashCode() == hashCode) {

                }
            }

            @Override
            public void onActivityDestroyed(Activity activity) {
                if (activity.hashCode() == hashCode) {

                }
            }
        });

        registrar.addRequestPermissionsResultListener(new PluginRegistry.RequestPermissionsResultListener() {
            @Override
            public boolean onRequestPermissionsResult(int id, String[] permissions, int[] grantResults) {
                if (id == PermissionHandler.PERMISSION_CODE) {
                    permissionHandler.onRequestPermissionsResult(id, permissions, grantResults);
                    return true;
                }
                return false;
            }
        });
        registrar.addActivityResultListener(new PluginRegistry.ActivityResultListener() {
            @Override
            public boolean onActivityResult(int requestCode, int responseCode, Intent intent) {
                if (requestCode == PermissionHandler.PERMISSION_CODE_IGNORE_BATTERY_OPTIMIZATIONS) {
                    permissionHandler.onActivityResult(requestCode, responseCode, intent);
                    return true;
                } else if (requestCode == PermissionHandler.PERMISSION_CODE_SYSTEM_ALERT_WINDOW) {
                    permissionHandler.onActivityResult(requestCode, responseCode, intent);
                } else if (requestCode == PermissionHandler.PERMISSION_CODE_REQUEST_INSTALL_PACKAGES) {
                    permissionHandler.onActivityResult(requestCode, responseCode, intent);
                }
                return false;
            }
        });
    }
}
