package com.qr.flutter_toolkit.package_info;

import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.os.Build;

import androidx.annotation.NonNull;

import java.util.HashMap;
import java.util.Map;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.PluginRegistry;

public class PackageInfoHandler implements MethodChannel.MethodCallHandler {
    private final PluginRegistry.Registrar registrar;

    public PackageInfoHandler(PluginRegistry.Registrar registrar) {
        this.registrar = registrar;
    }

    @Override
    public void onMethodCall(@NonNull MethodCall methodCall, @NonNull MethodChannel.Result result) {
        Context context = registrar.context();
        try {
            PackageInfo packageInfo = context.getPackageManager().getPackageInfo(context.getPackageName(), 0);
            switch (methodCall.method) {
                case "getPackageInfo":
                    Map<String, Object> map = new HashMap<>();
                    map.put("versionName", packageInfo.versionName);
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.P) {
                        map.put("versionCode", packageInfo.getLongVersionCode());
                    } else {
                        map.put("versionCode", packageInfo.versionCode);
                    }
                    map.put("packageName", packageInfo.packageName);
                    result.success(map);
                    break;
                case "getVersionName":
                    result.success(packageInfo.versionName);
                    break;
                case "getVersionCode":
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.P) {
                        result.success(packageInfo.getLongVersionCode());
                    } else {
                        result.success(packageInfo.versionCode);
                    }
                    break;
                case "getPackageName":
                    result.success(packageInfo.packageName);
                    break;
                default:
                    result.notImplemented();
                    break;
            }
        } catch (PackageManager.NameNotFoundException e) {
            result.error("NameNotFound", e.toString(), null);
            e.printStackTrace();
        }
    }
}
