package com.qr.flutter_toolkit.path_provider;

import android.os.Build;
import android.os.Environment;

import androidx.annotation.NonNull;

import java.io.File;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.PluginRegistry;
import io.flutter.util.PathUtils;

public class PathProviderHandler implements MethodChannel.MethodCallHandler {
    private final PluginRegistry.Registrar registrar;


    public PathProviderHandler(PluginRegistry.Registrar registrar) {
        this.registrar = registrar;
    }


    @Override
    public void onMethodCall(@NonNull MethodCall methodCall, @NonNull MethodChannel.Result result) {
        switch (methodCall.method) {
            case "getInternalFilesDirectory":
                result.success(getInternalFilesDirectory());
                break;
            case "getInternalCacheDirectory":
                result.success(getInternalCacheDirectory());
                break;
            case "getExternalFilesDirectory":
                result.success(getExternalFilesDirectory());
                break;
            case "getExternalCacheDirectory":
                result.success(getExternalCacheDirectory());
                break;
            case "getExternalMusicDirectory":
                result.success(getExternalMusicDirectory());
                break;
            case "getExternalMoviesDirectory":
                result.success(getExternalMoviesDirectory());
                break;
            case "getExternalPicturesDirectory":
                result.success(getExternalPicturesDirectory());
                break;
            case "getExternalDCIMDirectory":
                result.success(getExternalDCIMDirectory());
                break;
            case "getExternalDownloadsDirectory":
                result.success(getExternalDownloadsDirectory());
                break;
            case "getExternalDocumentsDirectory":
                result.success(getExternalDocumentsDirectory());
                break;
            case "getExternalPublicRootDirectory":
                result.success(getExternalPublicRootDirectory());
                break;
            case "getExternalPublicMusicDirectory":
                result.success(getExternalPublicMusicDirectory());
                break;
            case "getExternalPublicMoviesDirectory":
                result.success(getExternalPublicMoviesDirectory());
                break;
            case "getExternalPublicPicturesDirectory":
                result.success(getExternalPublicPicturesDirectory());
                break;
            case "getExternalPublicDCIMDirectory":
                result.success(getExternalPublicDCIMDirectory());
                break;
            case "getExternalPublicDownloadsDirectory":
                result.success(getExternalPublicDownloadsDirectory());
                break;
            case "getExternalPublicDocumentsDirectory":
                result.success(getExternalPublicDocumentsDirectory());
                break;
            default:
                result.notImplemented();
        }
    }

    private String getInternalCacheDirectory() {
        return PathUtils.getCacheDirectory(registrar.context());
    }

    private String getInternalFilesDirectory() {
        return PathUtils.getFilesDir(registrar.context());
    }

    private String getExternalCacheDirectory() {
        final File dir = registrar.context().getExternalCacheDir();
        if (dir == null) {
            return null;
        }
        return dir.getAbsolutePath();
    }

    private String getExternalFilesDirectory() {
        final File dir = registrar.context().getExternalFilesDir(null);
        if (dir == null) {
            return null;
        }
        return dir.getAbsolutePath();
    }

    private String getExternalMusicDirectory() {
        final File dir = registrar.context().getExternalFilesDir(Environment.DIRECTORY_MUSIC);
        if (dir == null) {
            return null;
        }
        return dir.getAbsolutePath();
    }

    private String getExternalMoviesDirectory() {
        final File dir = registrar.context().getExternalFilesDir(Environment.DIRECTORY_MOVIES);
        if (dir == null) {
            return null;
        }
        return dir.getAbsolutePath();
    }

    private String getExternalPicturesDirectory() {
        final File dir = registrar.context().getExternalFilesDir(Environment.DIRECTORY_PICTURES);
        if (dir == null) {
            return null;
        }
        return dir.getAbsolutePath();
    }

    private String getExternalDCIMDirectory() {
        final File dir = registrar.context().getExternalFilesDir(Environment.DIRECTORY_DCIM);
        if (dir == null) {
            return null;
        }
        return dir.getAbsolutePath();
    }

    private String getExternalDownloadsDirectory() {
        final File dir = registrar.context().getExternalFilesDir(Environment.DIRECTORY_DOWNLOADS);
        if (dir == null) {
            return null;
        }
        return dir.getAbsolutePath();
    }

    private String getExternalDocumentsDirectory() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
            final File dir = registrar.context().getExternalFilesDir(Environment.DIRECTORY_DOCUMENTS);
            if (dir == null) {
                return null;
            }
            return dir.getAbsolutePath();
        } else {
            return null;
        }
    }


    private String getExternalPublicRootDirectory() {
        return Environment.getExternalStorageDirectory().getAbsolutePath();
    }

    private String getExternalPublicMusicDirectory() {
        return Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_MUSIC).getAbsolutePath();
    }

    private String getExternalPublicMoviesDirectory() {
        return Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_MOVIES).getAbsolutePath();
    }

    private String getExternalPublicPicturesDirectory() {
        return Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_PICTURES).getAbsolutePath();
    }

    private String getExternalPublicDCIMDirectory() {
        return Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DCIM).getAbsolutePath();
    }

    private String getExternalPublicDownloadsDirectory() {
        return Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOWNLOADS).getAbsolutePath();
    }

    private String getExternalPublicDocumentsDirectory() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
            return Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOCUMENTS).getAbsolutePath();
        } else {
            return null;
        }
    }
}
