import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

class GPSLocation {
  static const MethodChannel _methodChannel =
      const MethodChannel("com.qr.flutter_toolkit/gps_location/methods");
  static const EventChannel _eventChannel =
      const EventChannel("com.qr.flutter_toolkit/gps_location/events");

  final double latitude;
  final double longitude;
  final int timestamp;
  final bool mocked;
  final double altitude;
  final double accuracy;
  final double bearing;
  final double speed;
  final double speedAccuracy;

  GPSLocation({
    @required this.latitude,
    @required this.longitude,
    @required this.timestamp,
    @required this.mocked,
    @required this.altitude,
    @required this.accuracy,
    @required this.bearing,
    @required this.speed,
    @required this.speedAccuracy,
  });

  factory GPSLocation.fromMap(Map<String, dynamic> map) {
    return GPSLocation(
      latitude: map["latitude"],
      longitude: map["longitude"],
      timestamp: map["timestamp"],
      mocked: map["mocked"],
      altitude: map["altitude"],
      accuracy: map["accuracy"],
      bearing: map["bearing"],
      speed: map["speed"],
      speedAccuracy: map["speedAccuracy"],
    );
  }

  @override
  String toString() {
    return '''GPSLocation{latitude:$latitude,
    longitude:$longitude,
    timestamp:$timestamp,
    mocked:$mocked,
    altitude:$altitude,
    accuracy:$accuracy,
    bearing:$bearing,
    speed:$speed,
    speedAcccuracy:$speedAccuracy,
    }''';
  }

  static Future<GPSLocation> get getLastKnownGPSLocation async {
    Map<String, dynamic> map =
        await _methodChannel.invokeMapMethod("getLastKnownGPSLocation");
    if (map == null) {
      return null;
    }
    return GPSLocation.fromMap(map);
  }

  static Future<GPSLocation> get getCurrentGPSLocation async {
    Map<String, dynamic> map =
        await _methodChannel.invokeMapMethod("getCurrentGPSLocation");
    if (map == null) {
      return null;
    }
    return GPSLocation.fromMap(map);
  }

  static Stream<GPSLocation> get getGPSLocationStream {
    return _eventChannel
        .receiveBroadcastStream()
        .map((value) => GPSLocation.fromMap(value.cast<String, dynamic>()));
  }
}
