import 'package:flutter/services.dart';
import 'dart:async';

class Permission {
  static const int permission_status_denied = 0;
  static const int permission_status_granted = 1;

  static const int permission_group_calendar = 0;
  static const int permission_group_camera = 1;
  static const int permission_group_contacts = 2;
  static const int permission_group_location = 3;
  static const int permission_group_microphone = 4;
  static const int permission_group_phone = 5;
  static const int permission_group_sensors = 6;
  static const int permission_group_sms = 7;
  static const int permission_group_storage = 8;
  static const int permission_group_ignore_battery_optimizations = 9;
  static const int permission_group_system_alert_window = 10;
  static const int permission_group_request_install_packages = 11;

  static const MethodChannel _methodChannel =
      const MethodChannel("com.qr.flutter_toolkit/permission/methods");

  static Future<int> checkPermissionsStatus(int group) async {
    return await _methodChannel.invokeMethod("checkPermissionsStatus", group);
  }

  static Future<int> requestPermission(int group) async {
    return await _methodChannel.invokeMethod("requestPermission", group);
  }

  static Future<Map<int, int>> requestPermissions(List<int> groups) async {
    Map<int, int> map = Map<int, int>();
    for (var group in groups) {
      var status = await requestPermission(group);
      map[group] = status;
    }
    return map;
  }

  static Future<bool> shouldShowRequestPermissionRationale(int group) async {
    return await _methodChannel.invokeMethod(
        "shouldShowRequestPermissionRationale", group);
  }

  static Future<bool> openAppSetting() async {
    return await _methodChannel.invokeMethod("openAppSetting");
  }
}
