import 'dart:async';
import 'dart:typed_data';
import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';

enum ImageFormat { JPEG, PNG, WEBP }

class VideoThumbnail {
  static const MethodChannel _channel = const MethodChannel('com.qr.flutter_toolkit/camera/methods');

  static Future<String> thumbnailFile({
    @required String video,
    @required ImageFormat imageFormat,
    String thumbnailPath,
    int maxHeightOrWidth = 0,
    int timeMs = 0,
    int quality = 50,
  }) async {
    assert(video != null && video.isNotEmpty);
    final reqMap = <String, dynamic>{
      'video': video,
      'path': thumbnailPath,
      'format': imageFormat.index,
      'maxhow': maxHeightOrWidth,
      'timeMs': timeMs,
      'quality': quality
    };
    return await _channel.invokeMethod('file', reqMap);
  }

  static Future<Uint8List> thumbnailData({
    @required String video,
    @required ImageFormat imageFormat,
    int maxHeightOrWidth = 0,
    int timeMs = 0,
    int quality = 50,
  }) async {
    assert(video != null && video.isNotEmpty);
    final reqMap = <String, dynamic>{
      'video': video,
      'format': imageFormat.index,
      'maxhow': maxHeightOrWidth,
      'timeMs': timeMs,
      'quality': quality
    };
    return await _channel.invokeMethod('data', reqMap);
  }
}
